@@ -219,6 +219,14 @@ const FrameStore = class {
         return null;
     }
 
+    updateURL(url) {
+        if ( typeof url !== 'string' ) { return; }
+        this.rawURL = url;
+        this.hostname = hostnameFromURI(url);
+        this.domain = domainFromHostname(this.hostname) || this.hostname;
+        this._cosmeticFilteringBits = undefined;
+    }
+
     getCosmeticFilteringBits(tabId) {
         if ( this._cosmeticFilteringBits !== undefined ) {
             return this._cosmeticFilteringBits;
@@ -234,6 +242,7 @@ const FrameStore = class {
                     .duplicate()
                     .fromTabId(tabId)
                     .setURL(this.rawURL)
+                    .setDocOriginFromURL(this.rawURL)
                     .setRealm('network')
                     .setType('specifichide')
                     .setFilter(staticNetFilteringEngine.toLogData())
@@ -253,6 +262,7 @@ const FrameStore = class {
                     .duplicate()
                     .fromTabId(tabId)
                     .setURL(this.rawURL)
+                    .setDocOriginFromURL(this.rawURL)
                     .setRealm('network')
                     .setType('generichide')
                     .setFilter(staticNetFilteringEngine.toLogData())
@@ -488,6 +498,8 @@ const PageStore = class {
         return this.frames.get(frameId) || null;
     }
 
+    // https://github.com/uBlockOrigin/uBlock-issues/issues/1858
+    //   Mind that setFrameURL() can be called from navigation event handlers.
     setFrameURL(details) {
         let { frameId, url, parentFrameId } = details;
         if ( frameId === undefined ) { frameId = 0; }
@@ -504,6 +516,9 @@ const PageStore = class {
         frameStore = FrameStore.factory(url, parentFrameId);
         this.frames.set(frameId, frameStore);
         this.frameAddCount += 1;
+        if ( url.startsWith('about:') ) {
+            frameStore.updateURL(this.getEffectiveFrameURL({ frameId }));
+        }
         if ( (this.frameAddCount & 0b111111) === 0 ) {
             this.pruneFrames();
         }
